<?php
/**
 * @copyright   Copyright (c) http://www.manadev.com
 * @license     http://www.manadev.com/license  Proprietary License
 */

namespace Manadev\ProductCollection\Resources\Facets\Swatch;

use Magento\Framework\DB\Select;
use Manadev\ProductCollection\Contracts\Facet;
use Manadev\ProductCollection\Contracts\FacetResource;
use Manadev\ProductCollection\Facets\Swatch\OptimizedFacet;
use Zend_Db_Expr;

class OptimizedFacetResource extends FacetResource
{

    /**
     * @param Select $select
     * @param Facet $facet
     * @return mixed
     */
    public function count(Select $select, Facet $facet)
    {
        /* @var $facet OptimizedFacet */
        $this->prepareSelect($select, $facet);
        $result = $this->getConnection()->fetchAll($select);
        return count($result) ? $result : false;
    }

    /**
     * @param Select $select
     * @param OptimizedFacet $facet
     */
    public function prepareSelect(Select $select, OptimizedFacet $facet)
    {
        $this->helperResource->clearFacetSelect($select);

        $selectedOptionIds = $facet->getSelectedOptionIds();

        if ($selectedOptionIds !== false) {
            return $this->_prepareSelectWithOptionIds($select, $facet, $selectedOptionIds);
        }

        return $this->_prepareSelect($select, $facet);
    }

    /**
     * @param Select $select
     * @param OptimizedFacet $facet
     * @param $selectedOptionIds
     */
    private function _prepareSelectWithOptionIds(Select $select, OptimizedFacet $facet, $selectedOptionIds)
    {
        $db = $this->getConnection();
        $storeId = $this->getStoreId();

        $selectedOptionIds = $facet->getSelectedOptionIds();

        $fields = [
            'sort_order' => new Zend_Db_Expr("`o`.`sort_order`"),
            'value' => new Zend_Db_Expr("`eav`.`value`"),
            'label' => new Zend_Db_Expr("COALESCE(`vs`.`value`, `vg`.`value`)"),
            'is_selected' => new Zend_Db_Expr("`eav`.`value` IN (" . implode(',', $selectedOptionIds) . ")"),
            'swatch_type' => new Zend_Db_Expr("COALESCE(`ss`.`type`, `sg`.`type`)"),
            'swatch' => new Zend_Db_Expr("COALESCE(`ss`.`value`, `sg`.`value`)"),
        ];

        return $select
            ->joinInner(['eav' => $this->getTable('catalog_product_index_eav')],
                "`eav`.`entity_id` = `e`.`entity_id` AND
                {$db->quoteInto("`eav`.`attribute_id` = ?", $facet->getAttributeId())} AND
                {$db->quoteInto("`eav`.`store_id` = ?", $storeId)}",
                ['count' => "COUNT(DISTINCT `eav`.`entity_id`)"]
            )
            ->joinInner(['o' => $this->getTable('eav_attribute_option')],
                "`o`.`option_id` = `eav`.`value`", null)
            ->joinInner(['vg' => $this->getTable('eav_attribute_option_value')],
                $db->quoteInto("`vg`.`option_id` = `eav`.`value` AND `vg`.`store_id` = ?", 0), null)
            ->joinLeft(['vs' => $this->getTable('eav_attribute_option_value')],
                $db->quoteInto("`vs`.`option_id` = `eav`.`value` AND `vs`.`store_id` = ?", $storeId), null)
            ->joinInner(['sg' => $this->getTable('eav_attribute_option_swatch')],
                $db->quoteInto("`sg`.`option_id` = `eav`.`value` AND `sg`.`store_id` = ?", 0), null)
            ->joinLeft(['ss' => $this->getTable('eav_attribute_option_swatch')],
                $db->quoteInto("`ss`.`option_id` = `eav`.`value` AND `ss`.`store_id` = ?", $storeId), null)
            ->columns($fields)
            ->group($fields);

    }

    /**
     * @param Select $select
     * @param OptimizedFacet $facet
     * @param $selectedOptionIds
     */
    private function _prepareSelect(Select $select, OptimizedFacet $facet)
    {
        $db = $this->getConnection();
        $storeId = $this->getStoreId();

        $fields = [
            'value' => new Zend_Db_Expr("DISTINCT(`eav`.`value`)"),
            'sort_order' =>0,
            'label' => new Zend_Db_Expr("COALESCE(`vs`.`value`, `vg`.`value`)"),
            'is_selected' => 0,
            'swatch_type' => new Zend_Db_Expr("COALESCE(`ss`.`type`, `sg`.`type`)"),
            'swatch' => new Zend_Db_Expr("COALESCE(`ss`.`value`, `sg`.`value`)"),
            'count' => 0
        ];
        return $select
            ->joinInner(['eav' => $this->getTable('catalog_product_index_eav')],
                "`eav`.`entity_id` = `e`.`entity_id` AND
                {$db->quoteInto("`eav`.`attribute_id` = ?", $facet->getAttributeId())} AND
                {$db->quoteInto("`eav`.`store_id` = ?", $storeId)}",
                null
            )
            ->joinInner(['o' => $this->getTable('eav_attribute_option')],
                "`o`.`option_id` = `eav`.`value`", null)
            ->joinInner(['vg' => $this->getTable('eav_attribute_option_value')],
                $db->quoteInto("`vg`.`option_id` = `eav`.`value` AND `vg`.`store_id` = ?", 0), null)
            ->joinLeft(['vs' => $this->getTable('eav_attribute_option_value')],
                $db->quoteInto("`vs`.`option_id` = `eav`.`value` AND `vs`.`store_id` = ?", $storeId), null)
            ->joinInner(['sg' => $this->getTable('eav_attribute_option_swatch')],
                $db->quoteInto("`sg`.`option_id` = `eav`.`value` AND `sg`.`store_id` = ?", 0), null)
            ->joinLeft(['ss' => $this->getTable('eav_attribute_option_swatch')],
                $db->quoteInto("`ss`.`option_id` = `eav`.`value` AND `ss`.`store_id` = ?", $storeId), null)
            ->columns($fields);

    }

    public function getFilterCallback(Facet $facet)
    {
        return $this->helperResource->dontApplyFilterNamed($facet->getName());
    }

    /**
     * Resource initialization
     *
     * @return void
     */
    protected function _construct()
    {
        $this->_setMainTable('catalog_product_index_eav');
    }
}